@@ -21,6 +21,8 @@ import re
 import sys
 from base64 import b64decode
 from collections import namedtuple
+from typing import List
+
 from math import ceil, floor
 
 from PyQt5.QtCore import (Qt, QSize, pyqtSignal)
@@ -31,10 +33,11 @@ from pre_workbench.bbuf_parsing import apply_grammar_on_bbuf
 from pre_workbench import configs, guihelper
 from pre_workbench.algo.range import Range
 from pre_workbench.configs import SettingsSection
-from pre_workbench.guihelper import setClipboardText, showWidgetDlg, getClipboardText
+from pre_workbench.guihelper import setClipboardText, showWidgetDlg, getClipboardText, qApp
 from pre_workbench.app import GlobalEvents
 from pre_workbench.controls.hexview_selheur import SelectionHelpers
 from pre_workbench.objects import ByteBuffer, parseHexFromClipboard, BidiByteBuffer
+from pre_workbench.rangetree import RangeTreeWidget
 from pre_workbench.util import PerfTimer
 
 group = SettingsSection('HexView2', 'Hex Editor', 'address', 'Address Styles')
@@ -74,6 +77,8 @@ class HexView2(QWidget):
 	parseResultsUpdated = pyqtSignal(list)
 	selectionChanged = pyqtSignal(object)
 
+	buffers: List[ByteBuffer]
+
 	def __init__(self, byteBuffer=None, annotationSetDefaultName="", options=dict(), optionsConfigKey="HexViewParams", project=None, formatInfoContainer=None):
 		super().__init__()
 		self.project = project
@@ -81,7 +86,6 @@ class HexView2(QWidget):
 		if self.formatInfoContainer: self.formatInfoContainer.updated.connect(self._formatInfoUpdated)
 		logging.debug("HexView - formatInfoContainer = %r", self.formatInfoContainer)
 		self.annotationSetDefaultName = annotationSetDefaultName
-		self.annotationSetName = None
 		self.buffers = list()
 		self.firstLine = 0
 		self.scrollY = 0
@@ -205,17 +209,13 @@ class HexView2(QWidget):
 					#TODO implement this
 					menu.addAction(name, lambda name=name: print(name))
 		else:
-			if self.project:
-				menu = ctx.addMenu("Load Annotation Set")
-				for name in self.project.getAnnotationSetNames():
-					menu.addAction(name, lambda name=name: self.loadAnnotations(name, self.selBuffer))
-				menu.addAction("New...", lambda: self.loadAnnotations(QInputDialog.getText(self, "Annotation Set", "Please enter a name for the new annotation set:")[0], self.selBuffer))
+			self._buildLoadAnnotationSetSubmenu(ctx, "Load Annotation Set", self.selBuffer, self.buffers[self.selBuffer].annotation_set_name)
+			self._buildParseBufferSubmenu(ctx, "Parse Buffer", self.selBuffer, self.buffers[self.selBuffer].fi_root_name)
+
+		menu = ctx.addMenu("Run Macro On Selection")
+		for container, macroName in qApp().find_macros_by_input_type("BYTE_ARRAY"):
+			menu.addAction(macroName, lambda c=container, name=macroName: c.getMacro(name).execute(self.getRangeBytes(self.selRange())))
 
-			if self.formatInfoContainer:
-				menu = ctx.addMenu("Parse Buffer")
-				for name in self.formatInfoContainer.definitions.keys():
-					menu.addAction(name, lambda name=name: self.applyFormatInfo(name, self.selBuffer))
-				#menu.addAction("New...", lambda: )
 
 	def _buildGeneralContextMenu(self, ctx):
 		ctx.addAction("Select all", lambda: self.selectAll())
@@ -223,19 +223,30 @@ class HexView2(QWidget):
 		ctx.addAction("Paste", lambda: self.setBuffer(parseHexFromClipboard()))
 		menu = ctx.addMenu("Paste as")
 		menu.addAction("Base64", lambda: self.setBuffer(ByteBuffer(b64decode(getClipboardText()))))
-		ctx.addAction("Clear Annotations" + (" on All Buffers" if len(self.buffers) > 0 else ""), lambda: self.clearRanges())
+		ctx.addAction("Clear Annotations" + (" on All Buffers" if len(self.buffers) > 1 else ""), lambda: self.clearRanges())
 
+		self._buildLoadAnnotationSetSubmenu(ctx, "Load Annotation Set" + (" on All Buffers" if len(self.buffers) > 1 else ""), None,
+											valueifsame(buf.annotation_set_name for buf in self.buffers))
+
+		self._buildParseBufferSubmenu(ctx, "Parse" + (" All Buffers" if len(self.buffers) > 1 else " Buffer"), None,
+											valueifsame(buf.fi_root_name for buf in self.buffers))
+
+	def _buildLoadAnnotationSetSubmenu(self, ctx, title, bufIdx, current):
 		if self.project:
-			menu = ctx.addMenu("Load Annotation Set" + (" on All Buffers" if len(self.buffers) > 0 else ""))
+			menu = ctx.addMenu(title)
 			for name in self.project.getAnnotationSetNames():
-				menu.addAction(name, lambda name=name: self.loadAnnotations(name))
-			menu.addAction("New...", lambda: self.loadAnnotations(
-				QInputDialog.getText(self, "Annotation Set", "Please enter a name for the new annotation set:")[0]))
+				act = menu.addAction(name, lambda name=name: self.loadAnnotations(name, bufIdx))
+				act.setCheckable(True)
+				act.setChecked(name == current)
+			menu.addAction("New...", lambda: self.loadAnnotations(QInputDialog.getText(self, "Annotation Set", "Please enter a name for the new annotation set:")[0], bufIdx))
 
+	def _buildParseBufferSubmenu(self, ctx, title, bufIdx, current):
 		if self.formatInfoContainer:
-			menu = ctx.addMenu("Parse" + (" All Buffers" if len(self.buffers) > 0 else " Buffer"))
+			menu = ctx.addMenu(title)
 			for name in self.formatInfoContainer.definitions.keys():
-				menu.addAction(name, lambda name=name: self.applyFormatInfo(name))
+				act = menu.addAction(name, lambda name=name: self.applyFormatInfo(name, bufIdx))
+				act.setCheckable(True)
+				act.setChecked(name == current)
 
 	def setDefaultAnnotationSet(self, name):
 		self.annotationSetDefaultName = name
@@ -244,27 +255,33 @@ class HexView2(QWidget):
 	def loadAnnotations(self, set_name, bufIdx=None):
 		for buf in self.buffers if bufIdx is None else [self.buffers[bufIdx]]:
 			buf.setRanges(buf.matchRanges(hasMetaKey='_sdef_ref'))
-			annotations = self.project.getAnnotations(set_name)
-			for rowid, start, end, meta_str in annotations:
-				meta = json.loads(meta_str)
-				if meta.get("deleted"): continue
-				meta['rowid'] = rowid
-				buf.addRange(Range(start=start, end=end, meta=meta))
+			if set_name is not None:
+				annotations = self.project.getAnnotations(set_name)
+				for rowid, start, end, meta_str in annotations:
+					meta = json.loads(meta_str)
+					if meta.get("deleted"): continue
+					meta['rowid'] = rowid
+					buf.addRange(Range(start=start, end=end, meta=meta))
+			buf.annotation_set_name = set_name
 		self.redraw()
-		self.annotationSetName = set_name
 
 	def storeAnnotation(self, range):
 		if not self.project: return
-		if not self.annotationSetName:
-			self.annotationSetName, ok = QInputDialog.getText(self, "Annotation Set", "Please enter a name for the new annotation set:")
-			if not ok or not self.annotationSetName: return
-		self.project.storeAnnotation(self.annotationSetName, range)
+		buf = self.buffers[self.selBuffer]
+		if not buf.annotation_set_name:
+			buf.annotation_set_name, ok = QInputDialog.getText(self, "Annotation Set", "Please enter a name for the new annotation set:")
+			if not ok or not buf.annotation_set_name: return
+		self.project.storeAnnotation(buf.annotation_set_name, range)
 
 	def clearRanges(self):
 		for buf in self.buffers:
 			buf.clearRanges()
+			buf.annotation_set_name = None
 		self.redraw()
 
+	def getRangeBytes(self, range):
+		return self.buffers[range.buffer_idx].getBytes(range.start, range.length())
+
 	def getRangeString(self, range, style=(" ","%02X")):
 		if isinstance(style, tuple):
 			return self.buffers[range.buffer_idx].toHex(range.start, range.length(), style[0], style[1])
@@ -334,10 +351,10 @@ class HexView2(QWidget):
 
 	def _fiTreeItemSelected(self, item, previous):
 		if item is None: return
-		range = item.data(0, Range.RangeRole)
+		range = item.data(0, RangeTreeWidget.RangeRole)
 		if range is not None:
 			self.selectRange(range, scrollIntoView=True)
-		#source = item.data(0, Range.SourceDescRole)
+		#source = item.data(0, RangeTreeWidget.SourceDescRole)
 		#if isinstance(source, structinfo.AbstractFI):
 			#self.on_data_selected.emit(source)
 
@@ -779,6 +796,11 @@ class HexView2(QWidget):
 		return ((firstBuf, firstOffset), (lastBuf, lastOffset))
 
 
+def valueifsame(iterable):
+	s = set(iterable)
+	return next(iter(s)) if len(s) == 1 else None
+
+
 def showHexView2Dialog(parent, title, content, ok_callback):
 	hexview = HexView2()
 	hexview.setBytes(content)
