@@ -13,6 +13,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+import hashlib
 import inspect
 import logging
 import os
@@ -21,23 +22,25 @@ import subprocess
 import tempfile
 import weakref
 import shlex
+from copy import deepcopy
 
 from PyQt5 import QtCore
 from PyQt5.QtCore import pyqtSignal, QUrl
 from PyQt5.QtGui import QDesktopServices, QColor
 from PyQt5.QtWidgets import QFileSystemModel, QTreeView, QWidget, QVBoxLayout, QAbstractItemView, QMenu, \
-	QAction, QListWidget, QListWidgetItem, QTreeWidget, QTreeWidgetItem, QTextEdit, QToolBar, QComboBox
+	QAction, QListWidget, QListWidgetItem, QTreeWidget, QTreeWidgetItem, QTextEdit, QToolBar, QComboBox, QMessageBox
 
 import pre_workbench.app
 from pre_workbench import configs
 from pre_workbench.algo.range import Range
-from pre_workbench.configs import getIcon
+from pre_workbench.configs import getIcon, SettingsField
 from pre_workbench.controls.genericwidgets import showSettingsDlg
+from pre_workbench.controls.scintillaedit import showScintillaDialog
 from pre_workbench.errorhandler import ConsoleWindowLogHandler
-from pre_workbench.guihelper import filledColorIcon, getMonospaceFont
+from pre_workbench.guihelper import filledColorIcon, getMonospaceFont, runProcessWithDlg, qApp, TODO
 from pre_workbench.app import navigate
+from pre_workbench.macros.macro import Macro
 from pre_workbench.rangetree import RangeTreeWidget
-from pre_workbench.structinfo.exceptions import parse_exception
 from pre_workbench.structinfo.parsecontext import AnnotatingParseContext
 from pre_workbench.windows.content.textfile import ScintillaEdit
 from pre_workbench.typeeditor import JsonView
@@ -242,7 +245,7 @@ class RangeTreeDockWidget(QWidget):
 
 	def _fiTreeItemSelected(self, item, previous):
 		if item is None: return
-		range = item.data(0, Range.RangeRole)
+		range = item.data(0, RangeTreeWidget.RangeRole)
 		hexView = self.lastHexView()
 		if range is not None and hexView is not None:
 			hexView.selectRange(range, scrollIntoView=True)
@@ -271,26 +274,22 @@ class RangeTreeDockWidget(QWidget):
 
 class DataInspectorWidget(QWidget):
 	defaultdef = """
-	DEFAULT union (ignore_errors=true, endianness=">"){
+	DataInspector union (ignore_errors=true, endianness=">"){
 		uint8 UINT8
 		int8 INT8
 		uint8_bin UINT8(show="{0:08b}")
-		BE union (endianness=">"){
-			uint16 UINT16
-			uint32 UINT32
-			int16_be INT16
-			int32_be INT32
-			float FLOAT
-			double DOUBLE
-		}
-		LE union (endianness="<"){
-			uint16 UINT16
-			uint32 UINT32
-			int16 INT16
-			int32 INT32
-			float FLOAT
-			double DOUBLE
-		}
+		uint16_BE UINT16(endianness=">")
+		uint32_BE UINT32(endianness=">")
+		int16_BE INT16(endianness=">")
+		int32_BE INT32(endianness=">")
+		float_BE FLOAT(endianness=">")
+		double_BE DOUBLE(endianness=">")
+		uint16_LE UINT16(endianness="<")
+		uint32_LE UINT32(endianness="<")
+		int16_LE INT16(endianness="<")
+		int32_LE INT32(endianness="<")
+		float_LE FLOAT(endianness="<")
+		double_LE DOUBLE(endianness="<")
 		ipv4 IPv4
 		ether ETHER
 	}
@@ -375,6 +374,125 @@ class RangeListWidget(QWidget):
 				# TODO ...on click: self.selectRange(d)
 
 
+class MacroListDockWidget(QWidget):
+	CONTAINER_ROLE = QtCore.Qt.UserRole + 100
+	MACRO_NAME_ROLE = QtCore.Qt.UserRole + 101
+	def __init__(self):
+		super().__init__()
+		self._initUI()
+		self._loadList()
+
+	def _initUI(self):
+		self.treeView = QTreeWidget()
+		self.treeView.setColumnCount(1)
+		self.treeView.setColumnWidth(1, 300)
+		self.treeView.headerItem().setText(0, "Name")
+		self.treeView.customContextMenuRequested.connect(self.customContextMenuRequested)
+		self.treeView.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
+		windowLayout = QVBoxLayout()
+		windowLayout.addWidget(self.treeView)
+		windowLayout.setContentsMargins(0,0,0,0)
+		self.setLayout(windowLayout)
+
+	def _loadList(self):
+		self.treeView.clear()
+		for title, container in qApp().macro_containers.items():
+			root = QTreeWidgetItem(self.treeView, [title])
+			root.setExpanded(True)
+			root.setData(0, MacroListDockWidget.CONTAINER_ROLE, container)
+			self._loadMacros(root, container)
+
+	def _loadMacros(self, root, container):
+		names = container.getMacroNames()
+		for name in names:
+			item = QTreeWidgetItem(root, [name])
+			item.setData(0, MacroListDockWidget.CONTAINER_ROLE, container)
+			item.setData(0, MacroListDockWidget.MACRO_NAME_ROLE, name)
+
+	def customContextMenuRequested(self, point):
+		item = self.treeView.itemAt(point)
+		ctx = QMenu("Context menu", self)
+		if item:
+			container = item.data(0, MacroListDockWidget.CONTAINER_ROLE)
+			macroname = item.data(0, MacroListDockWidget.MACRO_NAME_ROLE)
+			if macroname:
+				ctx.addAction("Execute", lambda: self.executeMacro(container.getMacro(macroname)))
+				if container.macrosEditable:
+					ctx.addAction("Edit code", lambda: self.editCode(container.getMacro(macroname)))
+					ctx.addAction("Preferences", lambda: self.editPreferences(container.getMacro(macroname)))
+					ctx.addAction("Delete macro", lambda: self.deleteMacro(container, macroname))
+				else:
+					ctx.addAction("View code", lambda: self.editCode(container.getMacro(macroname)))
+				ctx.addSeparator()
+				for title, target_container in qApp().macro_containers.items():
+					if target_container.macrosEditable and target_container != container:
+						ctx.addAction("Copy to " + title, lambda trg=target_container: self.copyMacro(container.getMacro(macroname), trg))
+			else:
+				if container.macrosEditable:
+					ctx.addAction("Create macro ...", lambda: self.createMacro(container))
+			ctx.exec(self.treeView.viewport().mapToGlobal(point))
+
+	def deleteMacro(self, container, macroname):
+		if QMessageBox.question(self, "Delete Macro", "Delete Macro \"" + macroname + "\"?", QMessageBox.Yes|QMessageBox.No) == QMessageBox.Yes:
+			container.deleteMacro(macroname)
+			self._loadList()
+
+	def executeMacro(self, macro):
+		if macro.input_type == Macro.TYPE_NONE:
+			macro.execute(None)
+		else:
+			TODO()
+
+	def editPreferences(self, macro):
+		result = showSettingsDlg([
+			SettingsField("name", "Name", "text", {}),
+			SettingsField("input_type", "Input Type", "select", {"options": zip(Macro.TYPES, Macro.TYPES)}),
+			SettingsField("output_type", "Output Type", "select", {"options": zip(Macro.TYPES, Macro.TYPES)}),
+		], {"name": macro.name, "input_type": macro.input_type, "output_type": macro.output_type}, title="Edit Preferences Of Macro \"" + macro.name + "\"", parent=self)
+		if not result: return
+		macro.name = result["name"]
+		macro.input_type = result["input_type"]
+		macro.output_type = result["output_type"]
+		macro.container.storeMacro(macro)
+		self._loadList()
+
+	def createMacro(self, container):
+		result = showSettingsDlg([
+			SettingsField("name", "Name", "text", {}),
+			SettingsField("input_type", "Input Type", "select", {"options": zip(Macro.TYPES, Macro.TYPES)}),
+			SettingsField("output_type", "Output Type", "select", {"options": zip(Macro.TYPES, Macro.TYPES)}),
+		], title="Create Macro ...", parent=self)
+		if not result: return
+		macro = Macro(container, result["name"], result["input_type"], result["output_type"], "", [], {}, None)
+		container.storeMacro(macro)
+		self.editCode(macro)
+		self._loadList()
+
+	def copyMacro(self, macro, target_container):
+		result = showSettingsDlg([
+			SettingsField("name", "Name", "text", {}),
+			SettingsField("input_type", "Input Type", "select", {"options": zip(Macro.TYPES, Macro.TYPES)}),
+			SettingsField("output_type", "Output Type", "select", {"options": zip(Macro.TYPES, Macro.TYPES)}),
+		], {"name": macro.name, "input_type": macro.input_type, "output_type": macro.output_type}, title="Copy Macro ...", parent=self)
+		if not result: return
+		new_macro = Macro(target_container, result["name"], result["input_type"], result["output_type"], macro.code, deepcopy(macro.options), deepcopy(macro.metadata), None)
+		target_container.storeMacro(new_macro)
+		self._loadList()
+
+	def editCode(self, macro):
+		verb = "Edit" if macro.container.macrosEditable else "View"
+		from PyQt5.Qsci import QsciLexerPython
+		res = showScintillaDialog(self, verb + " Code Of Macro \""+macro.name+"\"", macro.code, None,
+								  readonly=not macro.container.macrosEditable, lexer=QsciLexerPython())
+		if res:
+			if macro.container.macrosEditable:
+				macro.code = res
+				macro.container.storeMacro(macro)
+			hash = hashlib.sha256(res.encode('utf-8')).digest()
+			configs.updateMru("TrustedMacroHashes", hash, 255)
+
+
+
 class SelectionHeuristicsConfigWidget(QWidget):
 	HELPER_ROLE = QtCore.Qt.UserRole + 100
 
@@ -397,6 +515,12 @@ class SelectionHeuristicsConfigWidget(QWidget):
 		self.listView.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
 
 		from pre_workbench.controls.hexview_selheur import SelectionHelpers
+		self._updateList()
+		SelectionHelpers.updated.connect(self._updateList)
+
+	def _updateList(self):
+		from pre_workbench.controls.hexview_selheur import SelectionHelpers
+		self.listView.clear()
 		for helper, meta in SelectionHelpers.types:
 			item = QListWidgetItem(helper.__name__ )
 			item.setData(self.HELPER_ROLE, helper)
@@ -521,8 +645,11 @@ class ExtToolDockWidget(QWidget):
 		with tempfile.NamedTemporaryFile() as f:
 			f.write(self.selBytes)
 			f.flush()
-			result = subprocess.run(commandLine.format(shlex.quote(f.name)), stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
-			for line in result.stdout.decode('latin1').split('\n'):
+			args = shlex.split(commandLine)
+			args = [arg.format(f.name) for arg in args]
+			result = runProcessWithDlg("External command", "Running external command: " + args[0], self,
+									   args=args, stdin=subprocess.DEVNULL, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
+			for line in result['stdout'].decode('latin1').split('\n'):
 				root = QTreeWidgetItem(self.treeView)
 				root.setText(0, line)
 
